package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

public class ImagePreview extends JComponent {

	private BufferedImage dispImg;
	private BufferedImage srcImg;
	private Color bgColor;
	
	private boolean processing = false;
	
	/**
	 * Wofr auch immer...
	 */
	private static final long serialVersionUID = 1L;

	public ImagePreview() {
		setPreferredSize(new Dimension(176, 132));
	}
	
	private void processImage(BufferedImage img) {
		processing = true;
		repaint();
		
		dispImg = new BufferedImage(176, 132, BufferedImage.TYPE_INT_RGB);
		System.out.print("Bild geffnet: "+img.getWidth()+"x"+img.getHeight()+" pixels -> ");
		
		// Gre prfen
		if(img.getWidth() != 176 || img.getHeight() != 132) {
			// Gre ndern
			double xScale = (double)176.0 / img.getWidth();
			double yScale = (double)132.0 / img.getHeight();
			double scale = xScale < yScale ? xScale : yScale;
			
			// Seitenverhltnis passt, Gre anpassen
			System.out.println("Gre anpassen (Scale: "+scale+")");
			
			AffineTransform trans = new AffineTransform();
			trans.scale(scale, scale);
			
			Graphics2D g2D = dispImg.createGraphics();
			
			if(xScale != yScale) {
				g2D.setColor(bgColor);
				g2D.fillRect(0, 0, 176, 132);
				
				double tx = (176.0/scale-img.getWidth())/2.0;
				double ty = (132.0/scale-img.getHeight())/2.0;
				trans.translate(tx, ty);
			}
			
			g2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			g2D.drawImage(img, trans, null);
			g2D.dispose();
			
		} else {
			// Gre passt
			System.out.println("Format und Gre OK");
			
			dispImg = img;
		}
		
		processing = false;
		repaint();
	}
	
	public void paint(Graphics g) {
		
		if(dispImg != null && !processing) {
			g.drawImage(dispImg, (getWidth()-176)/2, (getHeight()-132)/2, null);
			
		} else {
			super.paint(g);
		}
		
		if(processing) {
			g.setColor(Color.BLACK);
			g.setFont(new Font("Sans-Serif", Font.PLAIN, 12));
			g.drawString("konvertiere...", 20, 75);
		}
	}
	
	public BufferedImage previewImage(BufferedImage img, Color bgColor) {
		this.bgColor = bgColor;
		srcImg = img;
		processImage(srcImg);
		
		return dispImg;
	}
	
	public BufferedImage setBgColor(Color bgColor) {
		this.bgColor = bgColor;
		if(srcImg != null) {
			processImage(srcImg);
			repaint();
		}
		
		return dispImg;
	}
}
