package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Vector;

import javax.comm.CommPortIdentifier;
import javax.comm.SerialPort;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Timer;

import util.ExampleFileFilter;

public class S65Picture extends JFrame implements ActionListener {

	private BufferedImage dispImg;
	private ExampleFileFilter filter;
	private File source;
	private Vector slideshow;
	private Timer slideshowTimer;
	private int slideshowIndex;
	
	private ImagePreview prev;
	private JButton bgColor;
	private JComboBox port;
	private JFileChooser chooser;
	
	/**
	 * Wofr auch immer...
	 */
	private static final long serialVersionUID = 1L;

	public S65Picture() {
		initGUI();
		initFrame();
	}

	private void initGUI() {
		Container c = getContentPane();
		c.setLayout(new BorderLayout(5,5));
		
		prev = new ImagePreview();
		JPanel p1 = new JPanel(new BorderLayout());
		p1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Preview"));
		p1.add(BorderLayout.CENTER, prev);
		c.add(BorderLayout.NORTH, p1);
		
		JPanel p2 = new JPanel(new BorderLayout(5,5));
		p2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "ffnen"));
		
		JButton browse = new JButton("ffnen");
		browse.setActionCommand("browse");
		browse.addActionListener(this);
		p2.add(BorderLayout.WEST, browse);
		
		JPanel p4 = new JPanel();
		p4.add(new JLabel("Hintergrund:"));
		
		bgColor = new JButton();
		bgColor.setBackground(Color.BLACK);
		bgColor.setPreferredSize(new Dimension(16,16));
		bgColor.setActionCommand("bgColor");
		bgColor.addActionListener(this);
		p4.add(bgColor);
		p2.add(BorderLayout.EAST, p4);
		
		c.add(BorderLayout.CENTER, p2);
		
		JPanel p3 = new JPanel(new BorderLayout(5,5));
		p3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Start"));
		
		Enumeration enu = CommPortIdentifier.getPortIdentifiers();
		Vector v = new Vector();
		while(enu.hasMoreElements()) {
			String name = ((CommPortIdentifier)enu.nextElement()).getName();
			if(name.startsWith("COM"))
				v.add(name);
		}
		String[] ports = new String[v.size()];
		for(int i=0; i<v.size(); i++)
			ports[i] = (String)v.get(i);
		
		port = new JComboBox(ports);
		p3.add(BorderLayout.CENTER, port);
		p3.add(BorderLayout.WEST, new JLabel("Port: "));
		
		JButton transmit = new JButton("Senden");
		transmit.setActionCommand("transmit");
		transmit.addActionListener(this);
		p3.add(BorderLayout.EAST, transmit);
		c.add(BorderLayout.SOUTH, p3);
		
		filter = new ExampleFileFilter();
	    filter.addExtension("jpg");
	    filter.addExtension("gif");
	    filter.addExtension("png");
	    filter.setDescription("Images");
	    
		chooser = new JFileChooser(".");
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
	    chooser.setFileFilter(filter);
	}
	
	private void initFrame() {
		setTitle("S65 Picture");
		pack();
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((d.width-getWidth())/2, (d.height-getHeight())/2);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setVisible(true);
	}

	private void transmitImage() {
		try {
			// Com Port ffnen
			CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(port.getSelectedItem().toString());
			SerialPort sport = (SerialPort)portId.open("S65Picture", 1000);
			sport.setSerialPortParams(460800, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			OutputStream out = sport.getOutputStream();
			
			System.out.print(port.getSelectedItem().toString()+" geffnet, Sende Bild... \nSende Bild... ");
			
			// Bild ans Display senden
			int p, r, g, b, c1, c2, n=0;
			byte[] buf = new byte[176*132*2];
			for(int y=0; y<132; y++) {
				for(int x=0; x<176; x++) {
					// in RGB splitten und auf 16-Bit runterrechnen (5-6-5)
					p = dispImg.getRGB(x, y);
					r = (p&0x00FF0000)>>19;
					g = (p&0x0000FF00)>>10;
					b = (p&0x000000FF)>>3;
					c1 = r<<3 | g>>3;
					c2 = g<<5 | b;
					
					buf[n++] = (byte)c1;
					buf[n++] = (byte)c2;
				}
			}
			
			out.write(buf, 0, n);
			
			out.close();
			sport.close();
			
			System.out.println("Fertig!");
			
		} catch(Exception ex) {
			System.err.println(ex);
		}
	}

	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		
		if(cmd == null) {
			try {
				if(slideshowIndex >= slideshow.size()) {
					slideshowIndex = 0;
				}
				BufferedImage img = ImageIO.read(new FileInputStream((File)slideshow.get(slideshowIndex++)));
				dispImg = prev.previewImage(img, bgColor.getBackground());
				transmitImage();
			} catch(Exception ex) {
				ex.printStackTrace();
			}
			
		} else if(cmd.equals("browse")) {
			int retVal = chooser.showOpenDialog(this);
			if(retVal == JFileChooser.APPROVE_OPTION) {
				source = chooser.getSelectedFile();
				try {
					if(!source.isDirectory()) {
						if(slideshowTimer != null)
							slideshowTimer.stop();
						BufferedImage img = ImageIO.read(new FileInputStream(source));
						dispImg = prev.previewImage(img, bgColor.getBackground());
					} else {
						slideshow = new Vector();
						slideshowIndex = 0;
						File[] f = source.listFiles();
						for(int i=0; i<f.length; i++) {
							if(!f[i].isDirectory() && filter.accept(f[i]))
								slideshow.add(f[i]);
						}
						if(slideshow.size() > 0) {
							BufferedImage img = ImageIO.read(new FileInputStream((File)slideshow.get(slideshowIndex++)));
							dispImg = prev.previewImage(img, bgColor.getBackground());
							transmitImage();
							slideshowTimer = new Timer(15000, this);
							slideshowTimer.start();
						}
					}
				} catch(Exception ex) {
					ex.printStackTrace();
				}
			}
			
		} else if(cmd.equals("bgColor")) {
			Color c = JColorChooser.showDialog(this, "Farbe whlen", bgColor.getBackground());
			bgColor.setBackground(c);
			dispImg = prev.setBgColor(c);
			
		} else if(cmd.equals("transmit")) {
			transmitImage();
		}
	}
	
	public static void main(String args[]) {
		new S65Picture();
	}
}
